#include "testlib.h"
#include <vector>
#include <cstdlib>

using namespace std;

const int N[] = { 3, 1000, 1000, 40000, 100000 };
const int T[] = { 3, 1, 1, 1, 20 };

const int C1 = 1e4;
const int C2 = 1e8;

inline int gcd(int a, int b) {
    while (b) {
        int y = a % b;
        a = b;
        b = y;
    }
    return a;
}

inline pair<int, int> normalize(int a, int b) {
    if (a < 0 || (a == 0 && b < 0)) {
        a = -a, b = -b;
    }
    int g = gcd(abs(a), abs(b));
    a /= g, b /= g;
    return make_pair(a, b);
}

int main(int argc, char **argv) {
	registerValidation(argc, argv);
	int group = atoi(validator.group().data());
	int n = inf.readInt(2, N[group], "n");
	inf.readSpace();
	int t = inf.readInt(1, T[group] , "t");
	inf.readEoln();
	vector<pair<int, int>> lines;
	vector<tuple<int, int, int>> rlines;
	for (int i = 0; i < n; i++) {
		int a = inf.readInt(-C1, C1, "a[" + vtos(i + 1) + "]");
		inf.readSpace();
		int b = inf.readInt(-C1, C1, "b[" + vtos(i + 1) + "]");
		inf.readSpace();
		ensuref(a != 0 || b != 0, "either a or b should be non-zero");
		int c = inf.readInt(-C2, C2, "c[" + vtos(i + 1) + "]");
		if (a < 0 || (a == 0 && b < 0)) {
			a = -a, b = -b, c = -c;
		}
		int g2 = gcd(gcd(abs(a), abs(b)), abs(c));
		rlines.push_back(make_tuple(a / g2, b / g2, c / g2));
		int g = gcd(abs(a), abs(b));
		lines.push_back(make_pair(a / g, b / g));
		inf.readEoln();
	}
	for (int i = 0; i < t; i++) {
		int u = inf.readInt(-C1, C1, "u[" + vtos(i + 1) + "]");
		inf.readSpace();
		int v = inf.readInt(-C1, C1, "v[" + vtos(i + 1) + "]");
		inf.readSpace();
		ensuref(u != 0 || v != 0, "either u or v should be non-zero");
		int w = inf.readInt(-C2, C2, "w[" + vtos(i + 1) + "]");
		if (group == 1) {
		    ensuref(u == 0, "lines-requests should be horizontal in the first group");
		}
	    if (u < 0 || (u == 0 && v < 0)) {
			u = -u, v = -v, w = -w;
		}
		int g = gcd(abs(u), abs(v));
		lines.push_back(make_pair(u / g, v / g));
		inf.readEoln();
	}
	sort(lines.begin(), lines.end());
	lines.resize(unique(lines.begin(), lines.end()) - lines.begin());
	ensuref(lines.size() == n + t, "No lines should be parallel!");
	sort(rlines.begin(), rlines.end());
	ensuref(unique(rlines.begin(), rlines.end()) - rlines.begin() == n, "No lines should be same!");
	inf.readEof();
	return 0;
}