#include "testlib.h"
#include <iostream>
#include <vector>
#include <algorithm>

using namespace std;

struct team
{
    int a, b;
};

bool cmp (const team &first, const team &second)
{
    return (first.a < second.a || first.a == second.a && first.b < second.b);
}

bool equal (const team &first, const team &second)
{
    return (first.a == second.a && first.b == second.b);
}

bool check_correct(int n, int m, vector<team> teams, int k, vector<int> colors)
{
    sort(colors.begin(), colors.end());
    for (int i = 0; i < k; ++i)
    {
        if (colors[i] < 1 || colors[i] > m)
        {
            return false;
        }
        if (i > 0 && colors[i] == colors[i - 1])
        {
            return false;
        } 
    }

    if (k > 3)
    {
        return false;
    }
    if (k == 3)
    {
        return true;
    }

    vector<team> new_teams;
    for (int i = 1; i < n; ++i)
    {
        if (equal(teams[i - 1], teams[i]))
        {
            new_teams.push_back(teams[i]);
        }
    }

    if (k == 2)
    {
        team new_team;
        new_team.a = colors[0];
        new_team.b = colors[1];
        bool correct = true;
        for (int i = 0; i < new_teams.size(); ++i)
        {
            if (equal(new_teams[i], new_team))
            {
                correct = false;
                break;
            }
        }
        return correct;
    }
    
    if (k == 1)
    {
        int color = colors[0];
        bool correct = true;
        for (int i = 0; i < new_teams.size(); ++i)
        {
            if (color == new_teams[i].a || color == new_teams[i].b)
            {
                correct = false;
                break;
            }
        }
        return correct;
    }

    return false;
}

int main(int argc, char * argv[])
{
    registerTestlibCmd(argc, argv);
    int n = inf.readInt();
    int m = inf.readInt();
    vector<team> teams;
    for (int i = 0; i < n; ++i)
    {
        int a = inf.readInt();
        int b = inf.readInt();
        team new_team;
        new_team.a = min(a, b);
        new_team.b = max(a, b);
        teams.push_back(new_team);
    }
    sort(teams.begin(), teams.end(), cmp);

    int jury_k = ans.readInt();
    if (jury_k > 3)
    {
        quitf(_fail, "Too many jury's colors");
    }

    vector<int> jury_colors;
    for (int i = 0; i < jury_k; ++i)
    {
        int color = ans.readInt();
        jury_colors.push_back(color);
    }
    //ans.readEoln();
    //ans.readEof();
    

    int part_k = ouf.readInt();
    if (part_k > 3)
    {
        quitf(_wa, "Participant's answer is not optimal: expected %d, found %d", jury_k, part_k);
    }
    
    vector<int> part_colors;
    for (int i = 0; i < part_k; ++i)
    {
        int color = ouf.readInt();
        part_colors.push_back(color);
    }
    //ouf.readEof();
    
    if (m == 2)
    {
        if (jury_k != -1)
        {
            quitf(_fail, "Jury's answer is incorrect: expected %d, found %d", -1, jury_k);
        }
        if (part_k != -1)
        {
            quitf(_wa, "Participant's answer is incorrect: expected %d, found %d", jury_k, part_k);
        }
        quitf(_ok, "It's impossible");
    }
    
    if (!check_correct(n, m, teams, jury_k, jury_colors))
    {
        quitf(_fail, "Jury's answer is incorrect");
    }

    if (!check_correct(n, m, teams, part_k, part_colors))
    {
        quitf(_wa, "Participant's answer is incorrect");
    }

    if (part_k > jury_k)
    {
        quitf(_wa, "Participant's answer is not optimal: expected %d, found %d", jury_k, part_k);
    }

    if (part_k < jury_k)
    {
        quitf(_fail, "Jury's answer is not optimal: expected %d, found %d", part_k, jury_k);
    }

    quitf(_ok, "%d colors", jury_k);
}

